import json
import os
from pathlib import Path
from datetime import datetime
from time import mktime, sleep

import pandas as pd
import pytest
from codeoceansdk.Pipeline import Pipeline
from codeoceansdk.Capsule import OriginalCapsuleInfo, UserPermission, EveryonePermission
from codeoceansdk.Computation import File

p = Path(__file__).parent
with open(p / Path("config.json")) as handle:
    test_parameters = json.load(handle)


def test_get_pipeline_runs():
    """
    Test previous runs. This pipeline has two successful runs and one unsuccessful run.
    """
    pipeline = Pipeline(
        id="fa743caa-f05a-48ba-b661-640887c1e250",
        domain=test_parameters["DOMAIN"],
        api_key=os.environ["API_KEY"],
    )
    output_runs = pipeline.get_pipeline_runs()
    output_runs = pd.DataFrame(output_runs)
    assert 3 == len(output_runs)
    has_results = output_runs[output_runs["has_results"]]
    assert 3 == len(
        has_results
    )  # this is wrong, this should be 2 instead of 3 but currently logs are being counted as "results" [sc-60178]


def test_get_pipeline():
    pipeline = Pipeline(
        id="a913d264-77ab-43b2-a5d4-103b48f576a6",
        domain=test_parameters["DOMAIN"],
        api_key=os.environ["API_KEY"],
    )
    pipeline.get_pipeline()
    # Original pipeline information not currently retained [sc-60179]
    # original_capsule = OriginalCapsuleInfo(id="fa743caa-f05a-48ba-b661-640887c1e250",
    #                                       major_version=0,
    #                                       minor_version=0,
    #                                       name="test_get_pipeline_computations",
    #                                       created=int(mktime(datetime.strptime("Jul 6, 2023 11:43 10",
    #                                                                            "%b %d, %Y %H:%M %S").timetuple())),
    #                                       public=False) #

    # assert pipeline.original_capsule == original_capsule
    assert datetime.fromtimestamp(pipeline.created) == datetime.strptime(
        "Sep 11, 2023 15:54 48", "%b %d, %Y %H:%M %S"
    )
    assert pipeline.status == "non-published"
    assert pipeline.owner == "1a091a69-17d2-4fd3-b8e6-cf0b091944b9"
    assert pipeline.name == "test_duplicated_pipeline"


def test_set_permissions():
    pipeline = Pipeline(
        id="a913d264-77ab-43b2-a5d4-103b48f576a6",
        domain=test_parameters["DOMAIN"],
        api_key=os.environ["API_KEY"],
    )
    pipeline.get_pipeline()

    # there's no actual way to check permissions, unfortunately, so we just have to run this and then switch back
    pipeline.set_capsule_permissions(
        users=[UserPermission("june@codeocean.com", "owner")],
        everyone=EveryonePermission("none"),
    )
    # there's no actual way to check permissions, unfortunately, so we just have to run this and then switch back
    pipeline.set_capsule_permissions(
        users=[UserPermission("june@codeocean.com", "owner")],
        everyone=EveryonePermission("viewer"),
    )


@pytest.mark.slow
def test_run_pipeline_computation():
    """Runs https://acmecorp-demo.codeocean.com/capsule/6558611/tree,
    attaches test dataset, capsule copies it and parameter 1 to the output folder."""
    pipeline = Pipeline(
        id="a913d264-77ab-43b2-a5d4-103b48f576a6",
        domain=test_parameters["DOMAIN"],
        api_key=os.environ["API_KEY"],
    )
    data_assets = [
        {"id": "c6f7b0ce-9d63-4b65-9b54-099d8e18c4e6", "mount": "test_params"}
    ]
    computation = pipeline.run_pipeline_computation(data_assets)
    max_iter = 10
    i = 0
    while computation.state != "completed":
        computation.get_computation()
        assert max_iter > i, "Max iterations exceeded."
        sleep(5)
        i += 1
    assert computation.end_status == "succeeded"
    computation.get_computation()
    assert computation.has_results
    computation_results = computation.list_computation_result_files()
    assert computation_results == [
        File(name="nextflow", path="nextflow", type="folder", size=None),
        File(name="output", path="output", type="file", size=306),
        File(name="test.tsv", path="test.tsv", type="file", size=27),
        File(
            name="test_parameter.txt", path="test_parameter.txt", type="file", size=15
        ),
    ]
